@@ -1,62 +1,56 @@
 # ABSTRACT: Search Complex Data Structures
 package Data::Seek;
 
-use Data::Seek::Data;
+use Data::Object::Class;
+use Data::Object::Signatures;
+
+use Data::Object qw(
+    reify
+);
+
+use Data::Object::Library qw(
+    HashObj
+    NumObj
+);
+
 use Data::Seek::Search;
 
-use Data::Object::Class;
-use Data::Object qw(:core);
+our $VERSION = '0.09'; # VERSION
 
-our $VERSION = '0.08'; # VERSION
+# ATTRIBUTES
 
-has 'cache' => (
-    is      => 'rw',
-    default => 0
+has data => (
+    is       => 'ro',
+    isa      => HashObj,
+    coerce   => method { reify($self) },
+    default  => method { {} },
+    required => 1,
 );
 
-has 'data' => (
-    is      => 'ro',
-    default => sub {{}}
+has ignore => (
+    is       => 'rw',
+    isa      => NumObj,
+    coerce   => method { reify($self) },
+    default  => 0,
+    required => 0,
 );
 
-has 'ignore' => (
-    is      => 'rw',
-    default => 0
-);
+# METHODS
+
+method search (Str @criteria) {
 
-sub search {
-    my ($self, @criteria) = @_;
-    my $cache  = $self->cache;
     my $data   = $self->data;
     my $ignore = $self->ignore;
 
-    Data::Seek::Exception->throw('Not Searchable')
-        unless UNIVERSAL::isa($data, 'HASH');
-
-    my $search;
-
-    if ($self->cache) {
-        if ($search = $self->{cached_search}) {
-            $search->criteria({});
-        }
-    }
-
-    $search //= Data::Seek::Search->new(
-        cache  => $cache,
-        ignore => $ignore,
-        data   => Data::Seek::Data->new($data),
+    my $place  = 0;
+    my $search = Data::Seek::Search->new(
+        criteria => { map { $_ => $place++ } @criteria },
+        data     => $data,
+        ignore   => $ignore,
     );
 
-    $search->criterion($_) for @criteria;
-
-    if ($self->cache) {
-        $self->{cached_search} = $search;
-    }
-    else {
-        $self->{cached_search} = undef;
-    }
-
     return $search->result;
+
 }
 
 1;
@@ -73,7 +67,7 @@ Data::Seek - Search Complex Data Structures
 
 =head1 VERSION
 
-version 0.08
+version 0.09
 
 =head1 SYNOPSIS
 
@@ -81,32 +75,15 @@ version 0.08
 
     my $hash   = {...};
     my $seeker = Data::Seek->new(data => $hash);
-    my $result = $seeker->search('*');
+    my $result = $seeker->search(...);
     my $data   = $result->data;
 
 =head1 DESCRIPTION
 
-Data::Seek is a module for traversing complex data structures. This module
-allows you to select specific node(s) in a hierarchical data structure using a
-criteria. A criteria is an expression consisting of one or more criterion. A
-criterion is the part of the criteria that is used to select node(s) to be
-returned in the result. Data::Seek is akin to L<Data::Dpath> but with fewer
-features and a simple node selection syntax. Data::Seek node traversal is
-extremely fast, and it can throw exception objects which explain, in detail,
-why a search failed. This is very useful internally, and externally when
-processing foreign data structures where you need to provide detailed errors
-explaining how to resolve the missing or malformed data nodes.
-
-=head1 CONCEPTS
-
-The follow is a short and simple overview of the strategy and syntax used by
-Data::Seek to query complex data strictures. The overall idea behind Data::Seek
-is to flatten/fold the data structure, preferably once, reduce it by applying a
-series patterns, then unflatten/unfold and operate on the new data structure.
-The introspection strategy is to flatten the data structure producing a
-non-hierarchical data structure where its keys represent endpoints (using
-dot-notation and colons to separate nested hash keys and array indices
-respectively) within the structure.
+Data::Seek is used for querying complex data structures. This module allows you
+to select and return specific node(s) in a hierarchical data structure using a
+simple and intuitive query syntax. The results can be returned as a list of
+values, or as a hash object in the same shape as the original.
 
 =head1 ENCODING
 
@@ -177,23 +154,23 @@ expressions) are converted into a series of regular expressions to be applied
 sequentially, filtering/reducing the endpoints and producing a data set of
 matching nodes or throwing an exception explaining the search failure.
 
-=head2 Node Expression
+=over 4
 
-    id
-    patient
-    medications
-
-The node expression is a part of a criterion, which preforms an exact match
-against a node in the data structure. It is a string which can contain letters,
-numbers, and/or underscores.
-
-=head2 Step Expression
+=item * B<Node Expression>
 
     my $result = $seeker->search(...);
 
     # given "id"
     { id => 12345 }
 
+The node expression is a part of a criterion, which preforms an exact match
+against a node in the data structure. It is a string which can contain letters,
+numbers, and/or underscores.
+
+=item * B<Step Expression>
+
+    my $result = $seeker->search(...);
+
     # given "patient.name.first"
     { patient => { name => { first => "Bob" } } }
 
@@ -205,7 +182,7 @@ more node expressions separated using the period character, which matches
 against nodes in the data structure. It is a string which can contain letters,
 numbers, and/or underscores, separated using periods.
 
-=head2 Index Expression
+=item * B<Index Expression>
 
     my $result = $seeker->search(...);
 
@@ -224,7 +201,7 @@ only match an array which has an index corresponding to the numeric portion of
 the suffix. It is a string which can contain letters, numbers, and/or
 underscores, suffixed with a semi-colon followed by a number.
 
-=head2 Iterator Expression
+=item * B<Iterator Expression>
 
     my $result = $seeker->search(...);
 
@@ -238,12 +215,12 @@ underscores, suffixed with a semi-colon followed by a number.
     { medications => [{ aceInhibitors => [{ pillCount => "#90" }] }] }
 
 The iteration expression is a criterion, or part of a criterion, having a node
-expressions preceded by an "at" character (ampersand) denoting that the node
-expression should match all nodes in the data structure which are mapped to
-array objects. It is a string which can contain letters, numbers, and/or
-underscores, preceded by a single ampersand character.
+expressions preceded by an "at" character denoting that the node expression
+should match all nodes in the data structure which are mapped to array objects.
+It is a string which can contain letters, numbers, and/or underscores, preceded
+by a single ampersand character.
 
-=head2 Wildcard Expression
+=item * B<Wildcard Expression>
 
     my $result = $seeker->search(...);
 
@@ -275,7 +252,7 @@ against a single node having a single "star" character match and represent one
 node expression. It is a string which can contain letters, numbers, underscores,
 and/or a single star character.
 
-=head2 Greedy-Wildcard Expression
+=item * B<Greedy-Wildcard Expression>
 
     my $result = $seeker->search(...);
 
@@ -301,23 +278,17 @@ matches against any multitude of nodes having a double "star" character match
 and represent one or more of any character. It is a string which can contain
 letters, numbers, underscores, and/or a double star character.
 
+=back
+
 =head1 ATTRIBUTES
 
-=head2 cache
-
-    $seeker->cache;
-    $seeker->cache(1);
-
-Encode the data structure and cache the result. Allows multiple queries to
-execute faster. Caching is disabled by default.
-
 =head2 data
 
     $seeker->data;
-    $seeker->data(Data::Seek::Data->new(...));
+    $seeker->data({...});
 
-The data structure to be introspected, must be a hash reference, blessed or not,
-which defaults to or becomes a L<Data::Seek::Data> object.
+The data structure to be introspected, must be a hash reference, which is
+coerced into a L<Data::Object::Hash> object.
 
 =head2 ignore
 
@@ -325,18 +296,29 @@ which defaults to or becomes a L<Data::Seek::Data> object.
     $seeker->ignore(1);
 
 Bypass exceptions thrown when a criterion is invalid or no data matches can be
-found.
+found. This attribute must be an integer, which is coerced into a
+L<Data::Object::Integer> object.
 
 =head1 METHODS
 
 =head2 search
 
-    my @criteria = ('id', 'person.name.*');
-    my $result   = $seeker->search(@criteria);
+    my $search = $seeker->search('id', 'person.name.*');
 
-Prepare a search object to use the supplied criteria and return a result
-object. Introspection is triggered when the result object is enacted. See
-L<Data::Seek::Search::Result> for usage information.
+Prepare a search object to use the supplied criteria and return a search
+object. Introspection is triggered when the result method is enacted. See
+L<Data::Seek::Search> for usage information.
+
+=head1 CONCEPT
+
+The follow is a short and simple overview of the strategy and syntax used by
+Data::Seek to query complex data structures. The overall idea behind Data::Seek
+is to flatten/fold the data structure, reduce it by applying a series patterns,
+then, unflatten/unfold and operate on the new data structure. The introspection
+strategy is to flatten the data structure producing a non-hierarchical data
+structure where its keys represent endpoints (using dot-notation and colons to
+separate (and denote) nested hash keys and array indices respectively) within
+the structure.
 
 =head1 AUTHOR
 
